package com.xiaoniu.niucai.common;


import java.util.Arrays;

public class ArrayUtils {

    /**
     * 抽取 数组 中指定部分内容, 返回新数组
     * @param src
     * @param start
     * @param end
     * @param <T>
     * @return
     */
    public static <T> T[] slice(T[] src, int start, int ... end) {
        if (null == src) {
            return null;
        }

        int from = start;
        int to = -1;
        if (end.length == 0) {
            to = src.length;
        } else {
            to = end[0];
        }

        if (from < 0) {
            from = 0;
        }

        if (from > src.length) {
            from = src.length;
        }

        if (from > to) {
            from = 0;
            to = 0;
        }

        return Arrays.copyOfRange(src, from, to);
    }

    /**
     * 合并两个数组, 返回新数组
     * @param a1
     * @param a2
     * @param <T>
     * @return
     */
    public static <T> T[] join(T[] a1, T[] a2) {
        if (a1 == null && a2 == null) {
            return null;
        }

        if (a1 == null) {
            return slice(a2, 0);
        }

        if (a2 == null) {
            return slice(a1, 0);
        }

        T[] res = Arrays.copyOf(a1, a1.length + a2.length);
        System.arraycopy(a2, 0, res, a1.length, a2.length);
        return res;
    }

    /**
     * 从数组指定位置删除一个元素, 返回新数组
     * @param src
     * @param index
     * @param <T>
     * @return
     */
    public static <T> T[] omit(T[] src, int index) {
        return join(slice(src, 0, index), slice(src,index + 1));
    }

    /**
     * indexOf
     * @param src
     * @param search
     * @param <T>
     * @return
     */
    public static <T> int indexOf(T[] src, T search) {
        return Arrays.asList(src).indexOf(search);
    }

    /**
     * 数组扩容
     * @param src
     * @param it
     * @param <T>
     * @return
     */
    public static <T> T[] concat(T[] src, T it) {
        T[] nw = Arrays.copyOf(src, src.length + 1);
        nw[src.length] = it;
        return nw;
    }

}
