package com.xiaoniu.niucai.common;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ListUtils {
    public static <T> List<T> getDeepCopy(List<T> src) {
        List<T> target = new ArrayList<>();
        for (int i = 0; i < src.size(); i ++) {
            target.add(null);
        }
        Collections.copy(target, src);
        return target;
    }

    /**
     * 笛卡尔积
     * @param src
     * @param <T>
     * @return
     */
    public static <T> List<List<T>> descartes(List<List<T>> src) {
        if (src == null || src.size() < 2) {
            return null;
        }

        List<List<T>> result = new ArrayList<>();
        List<T> srcA = src.get(0);
        if (src.size() == 2) {
            List<T> srcB = src.get(1);

            // 嵌套实现 cross
            for (T a : srcA) {
                for (T b : srcB) {
                    result.add(new ArrayList<T>(){{
                        add(a);
                        add(b);
                    }});
                }
            }

            return result;
        } else {
            // 去掉第一个, 剩余的继续迭代
            List<List<T>> srcCopy = getDeepCopy(src);   // 不影响原来 list
            srcCopy.remove(0);
            List<List<T>> rtnList = descartes(srcCopy);

            for (T a : srcA) {
                for (List<T> rtn : rtnList) {
                    List<T> tmp = new ArrayList<>();
                    tmp.add(a);
                    tmp.addAll(rtn);
                    result.add(tmp);
                }
            }
        }

        return result;
    }


    /**
     * 组合
     * 从 src 中抽取组合, 组成长度为 len 的集合
     * @param src
     * @param len
     * @param <T>
     * @return
     */
    public static <T> List<List<T>> combination(List<T> src, int len) {
        if (null == src || src.size() == 0) {
            return null;
        }

        if (src.size() < len) {
            return null;
        }

        // 递归
        return combinationRecur(src, len, new ArrayList<T>(), 0);
    }

    private static <T> List<List<T>> combinationRecur(List<T> src, int len, List<T> last, int index) {
        // 循环次数 = list 长度 - 剩余需要组装的长度
        int times = src.size() - (len - last.size()) + 1;

        List<List<T>> result = new ArrayList<>();
        for (int i = index; i < times; i ++) {
            List<T> current = getDeepCopy(last);
            current.add(src.get(i));

            if (current.size() < len) {
                result.addAll(combinationRecur(src, len, current, i + 1));
            } else {
                result.add(current);
            }
        }

        return result;
    }

}
