package com.xiaoniu.niucai.config;

import com.xiaoniu.niucai.task.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;

@Configuration
public class ScheduledConfig implements SchedulingConfigurer {

    @Autowired
    JobCronConfig jobCronConfig;

    @Autowired
    CrawlFootball crawlFootball;

    @Autowired
    CrawlBasketball crawlBasketball;

    @Autowired
    CrawlLottery crawlLottery;

    @Autowired
    CleanCrawlSpam cleanCrawlSpam;

    @Autowired
    OrderExpire orderExpire;

    @Autowired
    SportPrize sportPrize;

    @Autowired
    LotteryPrize lotteryPrize;

    @Autowired
    MatchException matchException;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();
        // 10 个线程
        taskScheduler.setPoolSize(jobCronConfig.getPoolSize());
        taskScheduler.initialize();
        scheduledTaskRegistrar.setTaskScheduler(taskScheduler);

        scheduledTaskRegistrar.addCronTask(crawlFootball, jobCronConfig.getCrawlFootball());
        scheduledTaskRegistrar.addCronTask(crawlBasketball, jobCronConfig.getCrawlBasketball());
        scheduledTaskRegistrar.addCronTask(crawlLottery, jobCronConfig.getCrawlLottery());
        scheduledTaskRegistrar.addCronTask(cleanCrawlSpam, jobCronConfig.getCleanCrawlSpam());
        scheduledTaskRegistrar.addCronTask(orderExpire, jobCronConfig.getOrderExpire());
        scheduledTaskRegistrar.addCronTask(sportPrize, jobCronConfig.getSportPrize());
        scheduledTaskRegistrar.addCronTask(lotteryPrize, jobCronConfig.getLotteryPrize());
        scheduledTaskRegistrar.addCronTask(matchException, jobCronConfig.getMatchException());
    }
}
