package com.xiaoniu.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xiaoniu.niucai.entity.TaBettingPlan;
import com.xiaoniu.niucai.entity.TaBettingPlanItem;
import com.xiaoniu.niucai.entity.TaBettingPlanSportDetail;
import com.xiaoniu.niucai.enums.StatusEnum;
import com.xiaoniu.niucai.service.ITaBettingPlanService;
import com.xiaoniu.niucai.mapper.TaBettingPlanItemMapper;
import com.xiaoniu.niucai.mapper.TaBettingPlanMapper;
import com.xiaoniu.niucai.mapper.TaBettingPlanSportDetailMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 投注方案 服务实现类
 * </p>
 *
 * @author admin
 * @since 2020-09-08
 */
@Service
public class TaBettingPlanServiceImpl extends ServiceImpl<TaBettingPlanMapper, TaBettingPlan> implements ITaBettingPlanService {

    @Autowired
    TaBettingPlanSportDetailMapper taBettingPlanSportDetailMapper;

    @Autowired
    TaBettingPlanItemMapper taBettingPlanItemMapper;

    @Override
    public TaBettingPlan getDetailOfPlanById(Integer planId) {
        TaBettingPlan taBettingPlan = getById(planId);
        if (null == taBettingPlan || StatusEnum.DELETED.getCode().equals(taBettingPlan.getStatus())) {
            return null;
        }

        List<TaBettingPlanItem> itemList = getBettingPlanItemsByPlan(taBettingPlan);
        taBettingPlan.setItemList(itemList);

        if (null != itemList && taBettingPlan.getIsSport()) {
            for (TaBettingPlanItem item: itemList) {
                List<TaBettingPlanSportDetail> detailList = getBettingPlanSprotsByItem(item);
                item.setDetailList(detailList);
            }
        }

        return taBettingPlan;
    }

    private List<TaBettingPlanSportDetail> getBettingPlanSprotsByItem(TaBettingPlanItem item) {
        QueryWrapper<TaBettingPlanSportDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("item_no", item.getItemNo());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("item_no");
        queryWrapper.orderByAsc("serial_no");

        return taBettingPlanSportDetailMapper.selectList(queryWrapper);
    }

    private List<TaBettingPlanItem> getBettingPlanItemsByPlan(TaBettingPlan taBettingPlan) {
        QueryWrapper<TaBettingPlanItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plan_id", taBettingPlan.getPlanId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("item_no");

        return taBettingPlanItemMapper.selectList(queryWrapper);
    }

    /**
     * p3, p5 直选
     * 支持每一位多选情况
     * @param item
     * @return
     */
    private int computeDirectlyAmount(TaBettingPlanItem item) {
        int res = 1; // 最差是 1 注
        for (String num: item.getFirstNums().split(",")) {
            res *= num.split(" ").length;
        }
        return res;
    }

}
