package com.xiaoniu.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xiaoniu.niucai.entity.*;
import com.xiaoniu.niucai.enums.StatusEnum;
import com.xiaoniu.niucai.service.ITaBettingPlanService;
import com.xiaoniu.niucai.service.ITaCustomerBettingService;
import com.xiaoniu.niucai.mapper.TaCustomerBettingItemMapper;
import com.xiaoniu.niucai.mapper.TaCustomerBettingMapper;
import com.xiaoniu.niucai.mapper.TaCustomerBettingSportMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 客户投注  服务实现类
 * </p>
 *
 * @author admin
 * @since 2020-09-09
 */
@Service
public class TaCustomerBettingServiceImpl extends ServiceImpl<TaCustomerBettingMapper, TaCustomerBetting> implements ITaCustomerBettingService {

    @Autowired
    ITaBettingPlanService iTaBettingPlanService;

    @Autowired
    TaCustomerBettingSportMapper taCustomerBettingSportMapper;

    @Autowired
    TaCustomerBettingItemMapper taCustomerBettingItemMapper;

    @Override
    public List<TaCustomerBetting> getAllUnOpenBettingsBy(TdLottery lottery, TaLotteryResult lotteryResult) {
        QueryWrapper<TaCustomerBetting> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("lottery_id", lottery.getLotteryId());
        queryWrapper.eq("issue_no", lotteryResult.getIssueNo());
        queryWrapper.eq("is_drawn", 1);
        queryWrapper.eq("is_open", 0);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());

        List<TaCustomerBetting> list = list(queryWrapper);
        if (null != list && list.size() > 0) {
            for (TaCustomerBetting betting : list) {
                if (lottery.getIsSport()) {
                    betting.setBettingSports(getBettingSportByBetting(betting));
                }
                betting.setItemList(getBettingItemByBetting(betting));
                betting.setBettingPlan(iTaBettingPlanService.getDetailOfPlanById(betting.getPlanId()));
            }
        }

        return list;
    }

    private List<TaCustomerBettingSport> getBettingSportByBetting(TaCustomerBetting betting) {
        QueryWrapper<TaCustomerBettingSport> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("betting_id", betting.getBettingId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("serial_no");

        return taCustomerBettingSportMapper.selectList(queryWrapper);
    }

    private List<TaCustomerBettingItem> getBettingItemByBetting(TaCustomerBetting betting) {
        QueryWrapper<TaCustomerBettingItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("betting_id", betting.getBettingId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("item_no");

        return taCustomerBettingItemMapper.selectList(queryWrapper);
    }
}
