package com.xiaoniu.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xiaoniu.niucai.common.BizUtils;
import com.xiaoniu.niucai.common.DateUtils;
import com.xiaoniu.niucai.common.StringUtils;
import com.xiaoniu.niucai.entity.*;
import com.xiaoniu.niucai.enums.StatusEnum;
import com.xiaoniu.niucai.mapper.TaLotteryResultMapper;
import com.xiaoniu.niucai.service.*;
import com.xiaoniu.niucai.mapper.TaLotteryResultDetailMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 数字彩开奖结果  服务实现类
 * </p>
 *
 * @author admin
 * @since 2020-08-27
 */
@Slf4j
@Service
public class TaLotteryResultServiceImpl extends ServiceImpl<TaLotteryResultMapper, TaLotteryResult> implements ITaLotteryResultService {

    @Autowired
    ITdLotteryOnSaleService iTdLotteryOnSaleService;

    @Autowired
    ITdLotteryService iTdLotteryService;

    @Autowired
    ITdLotteryPrizeService iTdLotteryPrizeService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaCustomerBettingItemService iTaCustomerBettingItemService;

    @Autowired
    TaLotteryResultDetailMapper taLotteryResultDetailMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void checkAndSetLotteryPrize() throws Exception {
        LocalDateTime yestoday = LocalDateTime.now().minusDays(1); // DateUtils.str2LocalDateTime("2020-10-26 00:00", "yyyy-MM-dd HH:mm"); //
        String today = yestoday.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        int weekDay = yestoday.getDayOfWeek().getValue();
        List<TdLotteryOnSale> onSaleList = iTdLotteryOnSaleService.getByWeekDay(weekDay);
        if (null == onSaleList || onSaleList.size() == 0) {
            log.info(today + "未找到有开售设置");
            return;
        }

        for (TdLotteryOnSale onSale : onSaleList) {
            String lotteryId = onSale.getLotteryId();
            log.info(String.format("彩种 %s %s 开奖, 开始检测中奖人员", lotteryId, today));

            // 彩种字典
            TdLottery lottery = iTdLotteryService.getById(lotteryId);

            // 中奖设置
            List<TdLotteryPrize> prizeSettings = iTdLotteryPrizeService.getByLottery(lottery);
            if (null == prizeSettings || prizeSettings.size() == 0) {
                log.error(String.format("彩种 %s 没有开奖相关设置", lotteryId));
                continue;
            }

            // 当前彩种的开奖结果
            TaLotteryResult lotteryResult = getLotteryResultBy(onSale.getLotteryId(), today);
            if (null == lotteryResult) {
                log.error(String.format("彩种 %s 未抓取到开奖结果", lotteryId));
                continue;
            }

            if (null == lotteryResult.getDetailList() || lotteryResult.getDetailList().size() == 0) {
                log.error(String.format("彩种 %s 未抓取到开奖明细结果", lotteryId));
                continue;
            }

            // 当前彩种的投注列表
            List<TaCustomerBetting> customerBettingList = iTaCustomerBettingService.getAllUnOpenBettingsBy(lottery, lotteryResult);
            if (null == customerBettingList || customerBettingList.size() == 0) {
                log.info(String.format("彩种 %s 本期无人投注", lotteryId));
                continue;
            }

            // 遍历校验中奖情况
            for (TaCustomerBetting customerBetting : customerBettingList) {
                try {
                    if (lottery.getIsDigit()) {
                        validLotteryCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    // continue;
                }
            }
        }
    }

    public TaLotteryResult getLotteryResultBy(String lotteryId, String openingDate) {
        QueryWrapper<TaLotteryResult> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("lottery_id", lotteryId);
        queryWrapper.eq("opening_date", openingDate);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());

        TaLotteryResult result = getOne(queryWrapper);
        if (null != result) {
            QueryWrapper<TaLotteryResultDetail> query2 = new QueryWrapper<>();
            query2.eq("result_no", result.getSerialNo());
            query2.eq("status", StatusEnum.NORMAL.getCode());
            query2.orderByAsc("serial_no");
            result.setDetailList(taLotteryResultDetailMapper.selectList(query2));
        }

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    private void validLotteryCustomer(TdLottery lottery, List<TdLotteryPrize> prizeSettings, TaLotteryResult lotteryResult, TaCustomerBetting customerBetting) throws Exception {
        TaBettingPlan taBettingPlan = customerBetting.getBettingPlan();
        if (null == taBettingPlan) {
            throw new Exception(String.format("ID %s 的投注没有投注方案", customerBetting.getBettingId()));
        }

        if (null == taBettingPlan.getItemList() || taBettingPlan.getItemList().size() == 0
                || null == customerBetting.getItemList() || customerBetting.getItemList().size() == 0) {
            throw new Exception(String.format("ID %s 的投注没有投注明细", customerBetting.getBettingId()));
        }

        int winTotalCharges = 0;
        for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
            // 是否中奖
            boolean isWinning = false;
            // 中奖金额
            int winAmount = 0;

            TaCustomerBettingItem bettingItem = getBettingItemMap(customerBetting.getItemList(), item);
            if (bettingItem == null) {
                throw new Exception(String.format("ID %s 的投注查询映射投注明细失败", customerBetting.getBettingId()));
            }

            // 排列三直选, 排列五 都是依靠比对数字位是否一致来决定是否中奖的
            if (lottery.getLotteryId().equals("p5")) {
                isWinning = BizUtils.checkPrizeNumStrict(lotteryResult.getResult().split(","), item.getFirstNums().split(","));
                if (isWinning) {
                    TaLotteryResultDetail resultDetail = lotteryResult.getDetailList().get(0);
                    if (null == resultDetail) {
                        throw new Exception(String.format("第 %s 期排列五结果明细未抓取", lotteryResult.getIssueNo()));
                    }

                    winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                }
            } else if (lottery.getLotteryId().equals("p3")) {
                String[] bingoNums = lotteryResult.getResult().split(",");
                if (bingoNums.length != 3) {
                    throw new Exception(String.format("第 %s 期排列三抓取结果不正确", lotteryResult.getIssueNo()));
                }

//                String level = "直选";
//                if ("组选3".equals(item.getPlayWay())) {
//                    level = "组选三";
//                } else if ("组选6".equals(item.getPlayWay())) {
//                    level = "组选六";
//                }

                String stdP0 = "直选";
                String stdP3 = "组选3";
                String stdP6 = "组选6";
                String level = StringUtils.ifNull(item.getPlayWay(), stdP0);

                TaLotteryResultDetail resultDetail = filterResultDetail(lotteryResult.getDetailList(), level);
                if (null == resultDetail) {
                    throw new Exception(String.format("第 %s 期排列三结果明细未抓取", lotteryResult.getIssueNo()));
                }

                // 直选是通过比对数字来校验中奖
                if (stdP0.equals(level)) {
                    isWinning = BizUtils.checkPrizeNumStrict(bingoNums, item.getFirstNums().split(","));
                    if (isWinning) {
                        winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                    }
                } else {
                    // 出现2次的号码
                    boolean hasTwiceNum = bingoNums[0].equals(bingoNums[1])
                            || bingoNums[0].equals(bingoNums[2])
                            || bingoNums[1].equals(bingoNums[2]);

                    // 如果存在出现2次的号码, 只有可能中组选三, 否则只有可能中组选6
                    if (!hasTwiceNum) {
                        if (stdP6.equals(level)) {
                            isWinning = item.getFirstNums().contains(bingoNums[0])
                                    && item.getFirstNums().contains(bingoNums[1])
                                    && item.getFirstNums().contains(bingoNums[2]);
                            if (isWinning) {
                                winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                            }
                        }
                    } else {
                        if (stdP3.equals(level)) {
                            // 如果是复式
                            if (item.getIsMulti()) {
                                isWinning = item.getFirstNums().contains(bingoNums[0])
                                        && item.getFirstNums().contains(bingoNums[1])
                                        && item.getFirstNums().contains(bingoNums[2]);
                                if (isWinning) {
                                    winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                                }
                            } else {
                                // 投注号
                                String[] buyNums = item.getFirstNums().split(",");
                                // 由小到大排序
                                Arrays.sort(bingoNums);
                                Arrays.sort(buyNums);

                                isWinning = BizUtils.checkPrizeNumStrict(bingoNums, buyNums);
                                if (isWinning) {
                                    winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                                }
                            }
                        }
                    }
                }

            } else {
                // 大乐透  双色球
                String firstNums = item.getFirstNums();
                if (StringUtils.isEmpty(firstNums)) {
                    throw new Exception(String.format("ID %s 的投注明细无前区内容", customerBetting.getBettingId()));
                }

                String firstDanNums = item.getFirstDan();

                // 前区中奖数
                int firstSameNums = BizUtils.getSameNums(
                        lotteryResult.getFirstResult().split(","),
                        firstNums.split(","),
                        StringUtils.isEmpty(firstDanNums) ? null : firstDanNums.split(",")
                );

                String secondNums = item.getSecondNums();
                if (StringUtils.isEmpty(secondNums)) {
                    throw new Exception(String.format("ID %s 的投注明细无后区内容", customerBetting.getBettingId()));
                }

                String secondDanNums = item.getSecondDan();

                // 后区中奖数
                int secondSameNums = BizUtils.getSameNums(
                        lotteryResult.getSecondResult().split(","),
                        secondNums.split(","),
                        StringUtils.isEmpty(secondDanNums) ? null : secondDanNums.split(",")
                );

                // 选择结果
                String curNums = String.format("%d-%d", firstSameNums, secondSameNums);

                TdLotteryPrize bingo = null;
                for(TdLotteryPrize prize : prizeSettings) {
                    if (StringUtils.isEmpty(prize.getBingoRule())) {
                        throw new Exception(String.format("彩种 %s 的中奖结果规则未维护", lottery.getLotteryId()));
                    }

                    if (prize.getBingoRule().contains(curNums)) {
                        bingo = prize;
                        break;
                    }
                }

                if (null != bingo) {
                    log.info(String.format("恭喜 %s 购买 %s 中了 %s", customerBetting.getCustomerId(), lottery.getLotteryId(), bingo.getLevel()));
                    isWinning = true;
                    bettingItem.setWinLevel(bingo.getLevel());
                    TaLotteryResultDetail resultDetail = filterResultDetail(lotteryResult.getDetailList(), bingo.getLevel());
                    if (null == resultDetail) {
                        throw new Exception(String.format("彩种 %s 的中奖结果明细未成功抓取", lottery.getLotteryId()));
                    }

                    winAmount = Math.round(Float.parseFloat(resultDetail.getMoney().replaceAll(",", "")) * 100);
                    // 是否追加
                    boolean isAdd = customerBetting.getIsAdd();
                    if (isAdd && !StringUtils.isEmpty(resultDetail.getXMoney())) {
                        winAmount += Math.round(Float.parseFloat(resultDetail.getXMoney().replaceAll(",", "")) * 100);
                    }
                }
            }

            winTotalCharges += winAmount;
            bettingItem.setIsOpen(true);
            bettingItem.setOpeningDate(DateUtils.today());
            bettingItem.setIsWinning(isWinning);
            bettingItem.setWinAmount(winAmount);
            iTaCustomerBettingItemService.updateById(bettingItem);
        }

        if (winTotalCharges > 0) {
            customerBetting.setIsWinning(true);
            customerBetting.setWinAmount(winTotalCharges * customerBetting.getBettingTimes());
        }
        customerBetting.setIsOpen(true);
        customerBetting.setOpeningDate(DateUtils.today());
        iTaCustomerBettingService.updateById(customerBetting);
    }

    private TaLotteryResultDetail filterResultDetail(List<TaLotteryResultDetail> detailList, String level) {
        for (TaLotteryResultDetail d: detailList) {
            if (d.getLevel().equals(level)) {
                return d;
            }
        }

        return null;
    }

    private TaCustomerBettingItem getBettingItemMap(List<TaCustomerBettingItem> bettingItemList, TaBettingPlanItem item) {
        for (TaCustomerBettingItem it : bettingItemList) {
            if (item.getItemNo().equals(it.getPlanItemNo())) {
                return it;
            }
        }
        return null;
    }
}
